# Generated by Django 4.2.6 on 2023-10-25 18:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("metadata", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datachange",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="functioncall",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="requestcall",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
